﻿CREATE PROCEDURE insert_resource_code(p_parentResourceCode TEXT, p_resourceCode TEXT, p_resourceName TEXT, p_tableName TEXT, p_userId INT(11))
BEGIN
    
    DECLARE v_resourceId INT(11);
    DECLARE v_authActionType_value INT(11);
    DECLARE finished INT DEFAULT 0;

    # There are 6 main authorization action types namely View, List, Get, Insert, Update, Delete
    DECLARE cur1 CURSOR FOR SELECT param.value
                            FROM  coreParameters AS param
                            WHERE param.keyCode='AUTH_ACTION_TYPE'
                                    AND status=1
                            ORDER BY param.orderIndex, param.value;
                           
    DECLARE CONTINUE HANDLER FOR NOT FOUND SET finished = 1;
 
    DROP TEMPORARY TABLE IF EXISTS t_tempTable;
    CREATE TEMPORARY TABLE t_tempTable (actionId INT(11));
   
    # insert our new resource code
    INSERT INTO authResources (parentResourceCode, resourceCode, resourceName, tableName, resourceType, orderIndex, status, createdUserId, createdDate) 
        VALUES (p_parentResourceCode, p_resourceCode, p_resourceName, p_tableName, 0, 0, 1, p_userId, NOW());
        
    # get and store new id of our resource code
    SET v_resourceId = LAST_INSERT_ID();

    OPEN cur1;
      read_loop: LOOP
        FETCH FROM cur1 INTO v_authActionType_value;
        IF finished THEN LEAVE read_loop; END IF;
       
        INSERT INTO authActions (resourceId, actionType, orderIndex, status, createdUserId, createdDate)
                    VALUES (v_resourceId, v_authActionType_value, 0, 1, p_userId, NOW() );
                
        INSERT INTO t_tempTable (actionId) VALUES ( LAST_INSERT_ID() );
      END LOOP;
    CLOSE cur1;
 
            
    # Give the user 6 action rights of new resource code
    INSERT INTO authUserRights (actionId, userId, recordType, groupActionId, status, createdUserId, createdDate)
        SELECT actionId, p_userId, 0, 0, 1, p_userId, NOW() FROM t_tempTable;
       
       
    DROP TEMPORARY TABLE IF EXISTS t_tempTable;
    SELECT v_resourceId; # For check
END;