﻿-- FUNCTION: adm_sel_auth_claims(integer)

-- DROP FUNCTION adm_sel_auth_claims(integer);

CREATE OR REPLACE FUNCTION adm_sel_auth_claims(
    "p_userId" integer)
    RETURNS TABLE(rec "authResources", rec2 "authActions", rec3 "authUserRights") 
    LANGUAGE 'plpgsql'
    COST 100
    VOLATILE 
    ROWS 1000
AS $BODY$

declare v_roleId integer;

BEGIN

    select "roleId" into v_roleId from "coreUsers" where "coreUsers"."userId" = "p_userId";

IF COALESCE(v_roleId,0)=0 THEN

    RETURN QUERY
    SELECT "authResources"
            ,"authActions"
            ,"authUserRights"
    FROM "authResources"
        LEFT JOIN "authActions" ON "authActions"."resourceId" = "authResources"."resourceId"
        LEFT JOIN "authUserRights" ON "authUserRights"."actionId" = "authActions"."actionId"
                    AND "authUserRights"."userId" = "p_userId"
    WHERE "authResources"."status" = 1
    ORDER BY "authResources"."orderIndex";

ELSE

    RETURN QUERY
    SELECT "authResources"
            ,"authActions"
            ,"roles"
    FROM "authResources"
        LEFT JOIN "authActions" ON "authActions"."resourceId" = "authResources"."resourceId"
INNER JOIN (
    SELECT * FROM "authTemplate" 
        INNER JOIN "authTemplateDetail" on "authTemplateDetail"."templateId"="authTemplate"."templateId"
    WHERE "authTemplate"."templateId"=v_roleId
       ) "roles" ON 1=1
    WHERE "authResources"."status" = 1
    ORDER BY "authResources"."orderIndex";

END IF;

END;
$BODY$;

    
--------------------------------------------------------------------------------


-- FUNCTION: adm_sel_auth_claims_schema()

-- DROP FUNCTION adm_sel_auth_claims_schema();

CREATE OR REPLACE FUNCTION adm_sel_auth_claims_schema(
    )
    RETURNS TABLE(rec "authResources", rec2 "authActions") 
    LANGUAGE 'sql'
    COST 100
    VOLATILE 
    ROWS 1000
AS $BODY$

SELECT "authResources"
        ,"authActions"
FROM "authResources"
    LEFT JOIN "authActions" ON "authActions"."resourceId" = "authResources"."resourceId"
ORDER BY "authResources"."orderIndex","authActions"."actionType";

$BODY$;


--------------------------------------------------------------------------------


-- FUNCTION: adm_sel_authtemplatedetails(integer, integer, integer, integer, integer)

-- DROP FUNCTION adm_sel_authtemplatedetails(integer, integer, integer, integer, integer);

CREATE OR REPLACE FUNCTION adm_sel_authtemplatedetails(
    "p_authTemplateId" integer,
    p_offset integer,
    p_limit integer,
    p_status integer,
    "p_requesterId" integer)
    RETURNS TABLE(rec "authTemplateDetail", asd integer) 
    LANGUAGE 'sql'
    COST 100
    VOLATILE 
    ROWS 1000
AS $BODY$

select "authTemplateDetail",1 from "authTemplateDetail"
        WHERE
        (COALESCE("p_authTemplateId" ,0)=0
            OR "authTemplateDetail"."authTemplateId" = "p_authTemplateId") 
            AND  ("p_status" IS NULL
            OR "authTemplateDetail"."status" = "p_status") AND "authTemplateDetail".status != -1

    ORDER BY "authTemplateDetail"."authTemplateId","authTemplateDetail"."resourceId","authTemplateDetail"."actionId"
$BODY$;

    
--------------------------------------------------------------------------------


-- FUNCTION: adm_sel_labels()

-- DROP FUNCTION adm_sel_labels();

CREATE OR REPLACE FUNCTION adm_sel_labels()
 RETURNS json
 LANGUAGE sql
AS $function$ SELECT
    ('{' || STRING_AGG(params.param, ',') || '}')::JSON
FROM
    (
    SELECT
        ('"' || "keyCode" || '":{"keyCode":"' || "keyCode" || '","translations": ' || ARRAY_TO_JSON(ARRAY_AGG(('{' || '"value":"' || "value" || '",' || '"languages":' || "translations"::JSONB::TEXT || '}')::JSON)) || '}') AS param
    FROM
        "coreParameters"
    GROUP BY
        "keyCode") params;

$function$
;


--------------------------------------


CREATE OR REPLACE FUNCTION adm_ins_transactionlogs("p_userId" integer, "p_serviceUrl" text, p_request text, p_response text, "p_currentClaims" text, "p_logType" integer, "p_logDateBegin" timestamp without time zone, "p_logDateEnd" timestamp without time zone, p_status integer, "p_statusCode" integer)
 RETURNS bigint
 LANGUAGE sql
AS $function$

INSERT INTO "transactionLogs" (
	"userId"
	,"serviceUrl"
	,"request"
	,"response"
	,"currentClaims"
	,"logType"
	,"logDateBegin"
	,"logDateEnd"
	,"status"
    ,"statusCode"
	)
VALUES (
	"p_userId"
	,"p_serviceUrl"
	,"p_request"::json
	,"p_response"::json
	,"p_currentClaims"::json
	,"p_logType"
	,"p_logDateBegin"
	,"p_logDateEnd"
	,"p_status"
    ,"p_statusCode"
	);

SELECT LASTVAL();$function$;


--------------------------------------


CREATE OR REPLACE FUNCTION insert_resource_code("p_parentResourceCode" text, "p_resourceCode" text, "p_resourceName" text, "p_tableName" text, "p_userId" bigint)
 RETURNS void
 LANGUAGE plpgsql
AS $function$

declare "_resourceId" bigint;
declare "_actionId" bigint;
declare "actions" record;

BEGIN
    
    INSERT INTO "authResources" ("parentResourceCode", "resourceCode", "resourceName", "tableName", "resourceType", "orderIndex", "status", "createdUserId") 
        VALUES ("p_parentResourceCode", "p_resourceCode", "p_resourceName", "p_tableName", 0, 0, 1, "p_userId");
    
    SELECT LASTVAL() into "_resourceId";
    
    FOR "actions" IN
      SELECT par.value::int4
           FROM  "coreParameters" par
           WHERE par."keyCode"='AUTH_ACTION_TYPE'
           ORDER BY par."orderIndex"
    LOOP
        INSERT INTO "authActions" ("resourceId", "actionType", "orderIndex", status, "createdUserId", "createdDate")
            SELECT "_resourceId","actions"."value", 0,1, "p_userId", now();
      
        SELECT LASTVAL() into "_actionId";
        
        INSERT INTO "authUserRights" ("actionId", "userId","recordType","groupActionId","status","createdUserId","createdDate")
            VALUES ("_actionId", "p_userId",0,0,1,"p_userId",NOW());
    END LOOP;

END;
$function$
;
